with Ada.Text_IO;
with Time_IO;

with Railroad.Station;

package body Railroad.Train is

   --+----------
   --  UTILITIES
   --+----------

   --  service utilities to pretty-print the list of Travelers
   --+ currently aboard the Train
   --+ (the correct layout depends on Capacity)
   procedure Aboard;
   procedure Report (S : Address_T);

   procedure Aboard is
      use Ada.Text_IO;
      How_Many : Passengers_T := Passengers_T'First;
   begin
      Put ("   [");
      for I in 1 .. Population loop
         if En_Route (I) then
            Put (Positive'Image (I));
            How_Many := How_Many + 1;
         end if;
      end loop;
      Put (" ]");
      case How_Many is
         when 0 =>
            Put_Line ("          |");
         when 1 =>
            Put_Line ("        |");
         when 2 =>
            Put_Line ("      |");
         when 3 =>
            Put_Line ("    |");
         when 4 =>
            Put_Line ("  |");
      when others =>
         --  never more than Capacity
         null;
      end case;
      How_Many := 0;
   end Aboard;

   procedure Report (S : Address_T) is
      use Ada.Text_IO;
      Passed : Duration;
   begin
      Passed := Clock - Start_Time;
      Put (" ");
      Time_IO.Put (Interval (Passed));
      Put (" " & String (Stops (S)));
      Aboard;
   end Report;

   --+---------------------------
   --  THE TRAIN PROCESS INSTANCE
   --+---------------------------

   task body The_Train is
      --  the Train is initially empty of Travelers
      Load : Passengers_T := 0;
      --  we assume that the travel time across Stations
      --+ are fixed and constant (really?)
      use Ada.Text_IO;
      use Railroad.Station;
   begin
      Put_Line ("   TIME CALLING @ " &
                "ABOARD       | TRAVELER FROM    TO");
      loop
         --  the Train runs along the circular line
         for S in Address_T loop
            Report (S);
            --  when stopping at a Station the Train first releases
            --+ the Travelers onboard who have arrived at destination
            --+ and then determines its remaining load
            Railway_Line (S).Stopping (Load);
            --+ subsequently the Train boards Travelers that depart
            --+ from that Station as the residual capacity allows
            Railway_Line (S).Boarding;
            --+ and finally the Train updates its count of onboard Travelers
            Railway_Line (S).Closedoors (Load);
            --+
            --+ at this point the Train runs along to the next Station
            --+ taking the journey time specified for that leg
            delay Travel_Times (S);
         end loop;
      end loop;
   end The_Train;

end Railroad.Train;
