with Ada.Text_IO;
with Time_IO;
with Ada.Calendar;

with Railroad;
with Railroad.Station;

package body Railroad.Traveler is

   --+---------------------------
   --  THE TRAVELER PROCESS CLASS
   --+---------------------------

   task body Traveler_T is
      Current_Home,
      Current_Away : Address_T;
      type Direction_T is (To, Back);
      --  to tell which way the Traveler travels
      Trip         : Direction_T := To;
      Passed       : Duration;
      --+------------------
      --  FACTORED SERVICE UTILITIES
      --+------------------
      procedure Report;
      procedure To_Work;
      procedure Back_Home;
      --+------------------
      procedure Report is
         use Ada.Text_IO, Ada.Calendar;
      begin
         --  we note the time elapsed so far
         Passed := Clock - Start_Time;
         Put (" ");
         Time_IO.Put (Railroad.Interval (Passed));
         Put_Line ("                        |    "
                   & Positive'Image (Id)
                   & "    "
                   & String (Stops (Current_Home))
                   & " "
                   & String (Stops (Current_Away)));
      end Report;
      --+------------------
      procedure To_Work is
      begin
         Trip := To;
         Current_Home := Home;
         Current_Away := Away;
      end To_Work;
      --+------------------
      procedure Back_Home is
      begin
         Trip := Back;
         Current_Home := Away;
         Current_Away := Home;
      end Back_Home;
      --+------------------
      use Railroad.Station;
   begin --  TRAVELER_T
      Current_Home := Home;
      Current_Away := Away;
      loop
         case Trip is
            when To =>
               --+-----------------------------
               --  the away-from-home direction
               --+-----------------------------
               Report;
               --  notify the departure station that the traveler
               --+ wants to go to a specific destination
               Railway_Line (Current_Home).Arrive (Current_Away, Id);
               --  at this point the traveler has arrived at destination
               --+ and prepares for return
               Back_Home;
            when Back =>
               --+------------------------
               --  the back-home direction
               --+------------------------
               Report;
               --  notify the departure station that the traveler
               --+ wants to go to a specific destination
               Railway_Line (Current_Home).Arrive (Current_Away, Id);
               --  at this point the traveler has arrived at destination
               --+ and prepares for the next journey
               To_Work;
         end case;
         --  the traveler stays some time at the current destination
         delay Duration (Interval);
      end loop;
   end Traveler_T;

end Railroad.Traveler;
